"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TilesetMerger = void 0;
const path_1 = __importDefault(require("path"));
const base_1 = require("../../base");
const base_2 = require("../../base");
const tilesets_1 = require("../../tilesets");
const tilesets_2 = require("../../tilesets");
const tilesets_3 = require("../../tilesets");
const tilesets_4 = require("../../tilesets");
const BoundingVolumes_1 = require("./BoundingVolumes");
/**
 * A class for merging multiple tilesets, to create a tileset that refers
 * to the merged ones as external tilesets.
 *
 * @internal
 */
class TilesetMerger {
    /**
     * Creates a new instance
     */
    constructor() {
        this.tilesetSources = [];
        this.tilesetSourceJsonFileNames = [];
        this.tilesetSourceIdentifiers = [];
    }
    /**
     * Merges the tileset from the specified sources into one tileset
     * that refers to the sources as external ones, and writes the
     * result into the given target.
     *
     * @param tilesetSourceNames - The tileset source names
     * @param tilesetTargetName - The tileset target name
     * @param overwrite - Whether target files should be overwritten
     * @returns A promise that resolves when the process is finished
     * @throws TilesetError When the input could not be processed
     * @throws TilesetError When the output already exists
     * and `overwrite` was `false`.
     */
    async merge(tilesetSourceNames, tilesetTargetName, overwrite) {
        // Create the sources and target
        for (const tilesetSourceName of tilesetSourceNames) {
            // Determine the name of the file that contains the tileset JSON data
            const tilesetSourceJsonFileName = tilesets_4.Tilesets.determineTilesetJsonFileName(tilesetSourceName);
            // Determine an "identifier" for the tileset source
            // (see `tilesetSourceIdentifiers` for details)
            let tilesetSourceDirectoryName;
            if (base_1.Paths.isDirectory(tilesetSourceName)) {
                tilesetSourceDirectoryName = path_1.default.basename(tilesetSourceName);
            }
            else {
                tilesetSourceDirectoryName = path_1.default.basename(path_1.default.dirname(tilesetSourceName));
            }
            const tilesetSourceIdentifier = TilesetMerger.createIdentifier(tilesetSourceDirectoryName, this.tilesetSourceIdentifiers);
            const tilesetSource = tilesets_2.TilesetSources.createAndOpen(tilesetSourceName);
            this.tilesetSources.push(tilesetSource);
            this.tilesetSourceJsonFileNames.push(tilesetSourceJsonFileName);
            this.tilesetSourceIdentifiers.push(tilesetSourceIdentifier);
        }
        this.tilesetTargetJsonFileName =
            tilesets_4.Tilesets.determineTilesetJsonFileName(tilesetTargetName);
        this.tilesetTarget = tilesets_3.TilesetTargets.createAndBegin(tilesetTargetName, overwrite);
        // Perform the actual merge
        this.mergeInternal();
        // Clean up by closing the sources and the target
        for (const tilesetSource of this.tilesetSources) {
            tilesetSource.close();
        }
        await this.tilesetTarget.end();
        this.tilesetSources.length = 0;
        this.tilesetSourceIdentifiers.length = 0;
        this.tilesetTarget = undefined;
        this.tilesetTargetJsonFileName = undefined;
    }
    /**
     * Internal method for `merge`
     */
    mergeInternal() {
        if (this.tilesetSources.length == 0 ||
            !this.tilesetTarget ||
            !this.tilesetTargetJsonFileName) {
            throw new base_2.DeveloperError("The sources and target must be defined");
        }
        // Parse the Tileset objects from all sources
        const tilesets = [];
        const length = this.tilesetSources.length;
        for (let i = 0; i < length; ++i) {
            const tilesetSource = this.tilesetSources[i];
            const tilesetSourceJsonFileName = this.tilesetSourceJsonFileNames[i];
            const tilesetJsonBuffer = tilesetSource.getValue(tilesetSourceJsonFileName);
            if (!tilesetJsonBuffer) {
                const message = `No ${tilesetSourceJsonFileName} found in input`;
                throw new tilesets_1.TilesetError(message);
            }
            const tileset = JSON.parse(tilesetJsonBuffer.toString());
            tilesets.push(tileset);
        }
        // Derive the information for the merged tileset
        const geometricError = TilesetMerger.getMergedGeometricError(tilesets);
        const box = TilesetMerger.getMergedBox(tilesets);
        const children = TilesetMerger.getChildren(tilesets, this.tilesetSourceIdentifiers, this.tilesetSourceJsonFileNames);
        const mergedTileset = {
            asset: {
                version: "1.1",
            },
            geometricError: geometricError,
            root: {
                boundingVolume: {
                    box: box,
                },
                refine: "ADD",
                geometricError: geometricError,
                children: children,
            },
        };
        // Write the merged tileset into the target
        const mergedTilesetJson = JSON.stringify(mergedTileset, null, 2);
        const mergedTilesetBuffer = Buffer.from(mergedTilesetJson);
        this.tilesetTarget.addEntry(this.tilesetTargetJsonFileName, mergedTilesetBuffer);
        // Copy the resources from the sources to the target
        this.copyResources();
    }
    /**
     * Copy the resources from the source tilesets into the target.
     *
     * This will obtain the entries of all sources, and add them
     * to the target, adding the `tilesetSourceIdentifier` to the
     * path for disambiguation.
     */
    copyResources() {
        if (this.tilesetSources.length == 0 || !this.tilesetTarget) {
            throw new base_2.DeveloperError("The sources and target must be defined");
        }
        const length = this.tilesetSources.length;
        for (let i = 0; i < length; ++i) {
            const tilesetSource = this.tilesetSources[i];
            const tilesetSourceIdentifier = this.tilesetSourceIdentifiers[i];
            const sourceKeys = tilesetSource.getKeys();
            for (const sourceKey of sourceKeys) {
                const value = tilesetSource.getValue(sourceKey);
                const targetKey = tilesetSourceIdentifier + "/" + sourceKey;
                if (value) {
                    this.tilesetTarget.addEntry(targetKey, value);
                }
            }
        }
    }
    /**
     * Creates an identifier that does not exist yet.
     *
     * If the given prefix is not yet contained in the given list,
     * then it is returned. Otherwise, it is made "unique" in an
     * unspecified way, and then returned.
     *
     * This does NOT add the new identifier to the given list!
     *
     * @param prefix - The prefix
     * @param existingIdentifiers - The existing identifiers
     * @returns The new identifier
     */
    static createIdentifier(prefix, existingIdentifiers) {
        let identifier = prefix;
        let counter = 0;
        for (;;) {
            if (!existingIdentifiers.includes(identifier)) {
                return identifier;
            }
            identifier = `${prefix}-${counter}`;
            counter++;
        }
    }
    //========================================================================
    // The following functions are ported from the `merge-tilesets` branch
    // at https://github.com/CesiumGS/3d-tiles-tools/blob/d7e76e59022fc5e5aa4b848730ec9f8f4dea6d4e/tools/lib/mergeTilesets.js
    // with slight modification of 'getChildren' for disambiguation
    // and updates to compute bounding boxes instead of bounding spheres
    static getChildren(tilesets, tilesetSourceIdentifiers, tilesetJsonFileNames) {
        const length = tilesets.length;
        const children = Array(length);
        for (let i = 0; i < length; ++i) {
            const tilesetJsonFileName = tilesetJsonFileNames[i];
            const tilesetSourceIdentifier = tilesetSourceIdentifiers[i];
            const tilesetUrl = base_1.Paths.join(tilesetSourceIdentifier, tilesetJsonFileName);
            children[i] = tilesets[i].root;
            children[i].content = {
                uri: tilesetUrl,
            };
            children[i].boundingVolume = {
                box: TilesetMerger.getBoundingBox(tilesets[i]),
            };
            delete children[i].children;
            delete children[i].transform;
        }
        return children;
    }
    static getMergedGeometricError(tilesets) {
        let geometricError = 0.0;
        const length = tilesets.length;
        for (let i = 0; i < length; ++i) {
            geometricError = Math.max(geometricError, tilesets[i].geometricError);
        }
        return geometricError;
    }
    static getBoundingBox(tileset) {
        const root = tileset.root;
        const boundingVolume = root.boundingVolume;
        const boundingVolumeBox = BoundingVolumes_1.BoundingVolumes.computeBoundingVolumeBoxFromBoundingVolume(boundingVolume);
        if (boundingVolumeBox === undefined) {
            throw new tilesets_1.TilesetError("No bounding volume found in root tile");
        }
        if (root.transform === undefined) {
            return boundingVolumeBox;
        }
        return BoundingVolumes_1.BoundingVolumes.transformBoundingVolumeBox(boundingVolumeBox, root.transform);
    }
    static getMergedBox(tilesets) {
        const length = tilesets.length;
        const boundingBoxes = Array(length);
        for (let i = 0; i < length; ++i) {
            boundingBoxes[i] = TilesetMerger.getBoundingBox(tilesets[i]);
        }
        const boundingBox = BoundingVolumes_1.BoundingVolumes.computeUnionBoundingVolumeBox(boundingBoxes);
        return boundingBox;
    }
}
exports.TilesetMerger = TilesetMerger;
//# sourceMappingURL=TilesetMerger.js.map